package edu.cmu.cs211.snake.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Iterator;

import javax.swing.JPanel;
import javax.swing.Timer;

public class SnakePanel extends JPanel {
	private static final long serialVersionUID = 6366150138501154425L;

	public static final int SQUARE_SIZE = 10;

	private boolean waitingforstart = true, gameover = false;

	private transient Game game;

	private Timer timer = new Timer (100, new ActionListener () {
		public void actionPerformed (ActionEvent e)
		{
			timer.setDelay (game.getUpdateSpeed ());
			game.tick ();
			if (game.isGameOver ())
				gameover = waitingforstart = true;
			repaint ();
		}
	});

	public KeyListener getKeyListener ()
	{
		return new KeyAdapter () {
			@Override
			public void keyPressed (KeyEvent event)
			{
				switch (event.getKeyCode ()) {
				case KeyEvent.VK_UP:
					game.getSnake ().setDirection (new Dimension (0, -1));
					break;
				case KeyEvent.VK_DOWN:
					game.getSnake ().setDirection (new Dimension (0, 1));
					break;
				case KeyEvent.VK_LEFT:
					game.getSnake ().setDirection (new Dimension (-1, 0));
					break;
				case KeyEvent.VK_RIGHT:
					game.getSnake ().setDirection (new Dimension (1, 0));
					break;
				case KeyEvent.VK_SPACE:

					if (waitingforstart)
						newGame ();
					break;
				}
			}
		};
	}

	public void newGame ()
	{
		game = new Game (new Dimension (getWidth () / SQUARE_SIZE, getHeight ()
		        / SQUARE_SIZE));
		timer.start ();
		waitingforstart = false;
	}

	@Override
	public void paintComponent (Graphics g)
	{
		super.paintComponent (g);
		Graphics2D g2 = (Graphics2D) g;

		g2.setRenderingHint (RenderingHints.KEY_ANTIALIASING,
		        RenderingHints.VALUE_ANTIALIAS_ON);

		if (waitingforstart || game == null) {
			// FIXME: how do I center text here?
			if (gameover == true) {
				g.drawString ("Game Over!", getWidth () / 2, getHeight () / 4);
			}
			g.drawString ("Press Space To Begin", getWidth () / 2,
			        getHeight () / 2);
			return;
		}

		g.clearRect (0, 0, getWidth (), getHeight ());

		// draw snake
		g.setColor (Color.RED);
		Iterator itr = game.getSnake ().getPoints ().iterator ();
		while (itr.hasNext ()) {
			Point p = (Point) itr.next ();
			g.fillOval (p.x * SQUARE_SIZE, p.y * SQUARE_SIZE, SQUARE_SIZE,
			        SQUARE_SIZE);
		}
		// draw food
		g.setColor (Color.BLUE);

		itr = game.getFoodManager ().getFoodLocations ().iterator ();
		while (itr.hasNext ()) {
			Point p = (Point) itr.next ();
			g.fillRect (p.x * SQUARE_SIZE, p.y * SQUARE_SIZE, SQUARE_SIZE,
			        SQUARE_SIZE);
		}

		g.drawString ("Score: " + game.getPoints (), 0, 12);
	}
}
